import pytest
from NekUpload.NekData.filters import *
from typing import Any

@pytest.mark.unit
@pytest.mark.parametrize("session_file,params,expected_params",[
    ("test.xml",
        {"OutputFile":"IntermediateFields","OutputFrequency": 100,"OutputStartTime":50.0},
        {"OutputFile":"IntermediateFields","OutputFrequency": 100,"OutputStartTime":50.0}
    ),
    ("test.xml",
        {"OutputFrequency": 100,"OutputStartTime":50.0},
        {"OutputFile":"test.xml","OutputFrequency": 100,"OutputStartTime":50.0}
    ),
    ("path/to/test.xml",
        {"OutputFrequency": 100,"OutputStartTime":50.0},
        {"OutputFile":"test.xml","OutputFrequency": 100,"OutputStartTime":50.0}
    ),
    ("test.xml",
        {"OutputFile":"IntermediateFields","OutputFrequency": 100},
        {"OutputFile":"IntermediateFields","OutputFrequency": 100,"OutputStartTime":0.0}
    ),
    ("test.xml",
        {"OutputFile":"IntermediateFields","OutputStartTime":50.0},
        {"OutputFile":"IntermediateFields","OutputFrequency": 1,"OutputStartTime":50.0}
    ),
    ("test.xml",
        {"OutputFile":"IntermediateFields","OutputFrequency": "100","OutputStartTime":50.0},
        {"OutputFile":"IntermediateFields","OutputFrequency": 100,"OutputStartTime":50.0}
    ),
    ("test.xml",
        {"OutputFile":"IntermediateFields","OutputFrequency": 100,"OutputStartTime":"50.0"},
        {"OutputFile":"IntermediateFields","OutputFrequency": 100,"OutputStartTime":50.0}
    ),
])
def test_nek_filter_checkpoint_factory(session_file:str,params:dict[str,Any],
                                    expected_params:dict[str,Any]):
    filter: CheckpointFilter = NekFilterFactory.get_filter("Checkpoint",params,session_file)

    assert filter.output_file == expected_params["OutputFile"], f"Expected OutputFile: {expected_params['OutputFile']}, but got: {filter.output_file}"
    assert filter.output_frequency == expected_params["OutputFrequency"], f"Expected OutputFrequency: {expected_params['OutputFrequency']}, but got: {filter.output_frequency}"
    assert filter.output_start_time == expected_params["OutputStartTime"], f"Expected OutputStartTime: {expected_params['OutputStartTime']}, but got: {filter.output_start_time}"

@pytest.mark.unit
@pytest.mark.parametrize("session_file,params,expected_params",[
    ("test.xml",
        {"OutputFile":"DragLift","OutputFrequency":10,"Boundary":"B[1,2]","PivotPoint":[1.0,.1,0.0]},
        {"OutputFile":"DragLift","OutputFrequency":10,"Boundary":"B[1,2]","PivotPoint":(1.0,.1,0.0)},
    ),
    ("test.xml",
        {"OutputFrequency":10,"Boundary":"B[1,2]","PivotPoint":[1.0,.1,0.0]},
        {"OutputFile":"test.xml","OutputFrequency":10,"Boundary":"B[1,2]","PivotPoint":(1.0,.1,0.0)},
    ),
    ("path/to/test.xml",
        {"OutputFrequency":10,"Boundary":"B[1,2]","PivotPoint":[1.0,.1,0.0]},
        {"OutputFile":"test.xml","OutputFrequency":10,"Boundary":"B[1,2]","PivotPoint":(1.0,.1,0.0)},
    ),
    ("test.xml",
        {"OutputFile":"DragLift","Boundary":"B[1,2]","PivotPoint":[1.0,.1,0.0]},
        {"OutputFile":"DragLift","OutputFrequency":1,"Boundary":"B[1,2]","PivotPoint":(1.0,.1,0.0)},
    ),
    ("test.xml",
        {"OutputFile":"DragLift","OutputFrequency":10,"PivotPoint":[1.0,.1,0.0]},
        {"OutputFile":"DragLift","OutputFrequency":10,"Boundary":"","PivotPoint":(1.0,.1,0.0)},
    ),
    ("test.xml",
        {"OutputFile":"DragLift","OutputFrequency":10,"Boundary":"B[1,2]"},
        {"OutputFile":"DragLift","OutputFrequency":10,"Boundary":"B[1,2]","PivotPoint":(0.0,0.0,0.0)},
    ),
    ("test.xml",
        {"OutputFile":"DragLift","OutputFrequency":"10","Boundary":"B[1,2]"},
        {"OutputFile":"DragLift","OutputFrequency":10,"Boundary":"B[1,2]","PivotPoint":(0.0,0.0,0.0)},
    ),
])
def test_nek_filter_aeroforces_factory(session_file:str,params:dict[str,Any],
                                    expected_params:dict[str,Any]):
    filter: AerodynamicForcesFilter = NekFilterFactory.get_filter("AeroForces",params,session_file)

    assert filter.output_file == expected_params["OutputFile"], f"Expected OutputFile: {expected_params['OutputFile']}, but got: {filter.output_file}"
    assert filter.output_frequency == expected_params["OutputFrequency"], f"Expected OutputFrequency: {expected_params['OutputFrequency']}, but got: {filter.output_frequency}"
    assert filter.boundary == expected_params["Boundary"]
    assert filter.pivot_point == expected_params["PivotPoint"]

@pytest.mark.unit
@pytest.mark.parametrize("session_file,params,expected_params",[
    ("test.xml",
        {"OutputFile":"TimeValues","OutputFrequency":10,"Line":(3,1,0.5,0,3,0.5,0)},
        {"OutputFile":"TimeValues","OutputFrequency":10,"Line": (3,1,.5,0,3,.5,0),
            "OutputOneFile": False,"OutputPlane":-1,"WaveSpace":False,"Points":(),"Box": ()},
    ),
])
def test_nek_filter_history_factory(session_file:str,params:dict[str,Any],
                                    expected_params:dict[str,Any]):
    filter: HistoryPointsFilter = NekFilterFactory.get_filter("History",params,session_file)

    assert filter.output_file == expected_params["OutputFile"], f"Expected OutputFile: {expected_params['OutputFile']}, but got: {filter.output_file}"
    assert filter.output_frequency == expected_params["OutputFrequency"], f"Expected OutputFrequency: {expected_params['OutputFrequency']}, but got: {filter.output_frequency}"
    assert filter.line == expected_params["Line"], f"Expected Line: {expected_params['Line']}, but got: {filter.line}"
    assert filter.points == expected_params["Points"], f"Expected Points: {expected_params['Points']}, but got: {filter.points}"
    assert filter.output_one_file == expected_params["OutputOneFile"], f"Expected OutputOneFile: {expected_params['output_one_file']}, but got: {filter.output_one_file}"
    assert filter.output_plane == expected_params["OutputPlane"], f"Expected OutputPlane: {expected_params['output_plane']}, but got: {filter.output_plane}"
    assert filter.wave_space == expected_params["WaveSpace"], f"Expected WaveSpace: {expected_params['WaveSpace']}, but got: {filter.wave_space}"
    assert filter.box == expected_params["Box"], f"Expected Box: {expected_params['Box']}, but got: {filter.box}"