from NekUpload.testutils import NekTestDataset,NekTestGeometryComposite,NekTestSessionExpansion
import NekUpload.testutils
from NekUpload.NekData.data_type import SolverType,Elements
from NekUpload.NekData.expansions import ExpansionDefinition,ModifiedExpansionFactory
from pathlib import Path

def abs_path(file: str) -> str:
    absolute_path = Path(__file__).resolve().parent / file
    return str(absolute_path)

_EXPANSION_FACTORY = ModifiedExpansionFactory() 

@NekUpload.testutils.dataset
@NekUpload.testutils.geometry_composite_info(
    geom_info=NekTestGeometryComposite(
        composite_id_to_definition_map= {0:"Q[22-47]", 1:"T[0-21]", 2:"E[0-1]", 3:"E[2-5]",
            4:"E[45]", 5:"E[56]", 6:"E[67]", 7:"E[79]", 8:"E[88]",9:"E[94-95]",
            10:"E[84,75,69,62,51,40,30,20,6]"
        },
        dim=2,
        description="Only consists of 2D elements"
    )
)
@NekUpload.testutils.session_expansion_info(
    exp_info=NekTestSessionExpansion(
        composite_id_to_expansion_definitions_map={
            0: [_EXPANSION_FACTORY.get_expansion(Elements.QUAD,6,("u","v"))],
            1: [_EXPANSION_FACTORY.get_expansion(Elements.TRI,7,("u","v"))],
        },
        description="Basic test of MODIFIED expansions"
    )
)
def ADR_2D_TriQuad():
    return NekTestDataset(
        solver_type=SolverType.ADR_SOLVER,
        session=abs_path("ADR_2D_TriQuad.xml"),
        geometry=abs_path("ADR_2D_TriQuad.nekg"),
        output=abs_path("ADR_2D_TriQuad.fld"),
        checkpoints=[abs_path("ADR_2D_TriQuad_0.chk")],
        filters=[],
        boundary_conditions=[]
    )

@NekUpload.testutils.dataset
@NekUpload.testutils.geometry_composite_info(
    geom_info=NekTestGeometryComposite(
        composite_id_to_definition_map= {0:"A[0-65]", 1:"F[211,217,222,227,232,236,240,245,249]",
            2:"F[4,0,17,13,86,82,93,89,126,122,139,135,212,228,241,266,269,302,305,334,337]",
            3:"F[133,129,146,142,157,153,164,160,177,173,190,186,250,246,242,336,339,346,348,356,358]",
            4:"F[184,180,197,193,106,102,113,109,55,51,68,64,251,238,224,350,353,318,321,284,288]",
            5:"F[62,58,75,71,35,31,42,38,10,7,23,20,225,220,215,286,290,272,276,258,262]", 
            6:"F[263,270,277,282,289,294,299,306,311,316,322,326,332,338,344,349,354,359]",
            7:"H[66-75]", 8:"R[76-111]",9:"P[112-180]"
        },
        dim=3,
        description="Only consists of 3D elements"
    )
)
def ADR_3D_AllElmt():
    return NekTestDataset(
        solver_type=SolverType.ADR_SOLVER,
        session=abs_path("ADR_3D_AllElmt.xml"),
        geometry=abs_path("ADR_3D_AllElmt.nekg"),
        output=abs_path("ADR_3D_AllElmt.fld"),
        checkpoints=[abs_path("ADR_3D_AllElmt_0.chk"),abs_path("ADR_3D_AllElmt_1.chk")],
        filters=[],
        boundary_conditions=[]
    )

@NekUpload.testutils.dataset
@NekUpload.testutils.geometry_composite_info(
    geom_info=NekTestGeometryComposite(
        composite_id_to_definition_map= {100:"F[92,102,104,5,88,36,26]", 
            101:"F[205,217,117,219,240,180,258,200,125,248,252,284,255,189]",
            102:"F[91,15,70,93,69,87,61,19,75,58,99,101,55,94,25,96,89,64,35,46,50,79,103,72,81,22,4,52,253,278,220,277,250,286,152,113,246,181,184,279,186,262,105,241,227,144,267,260,206,281,244,259,229,133,164,177,249,237,274,285,271,242]",
            103:"F[98,71,51,14,68,45,95]",
            104:"F[280,185,106,210,192,270,139,245,223,238,111,282,153,275,130,172]",
            105:"A[0-41]",106:"A[42-116]"
        },
        dim=3,
        description="Curved cylinder: only consists of 3D elements, mapping starts at 100, rather than 0"
    )
)
def ADR_3D_stacked_cylinders_curved():
    return NekTestDataset(
        solver_type=SolverType.ADR_SOLVER,
        session=abs_path("ADR_3D_stacked_cylinders_curved.xml"),
        geometry=abs_path("ADR_3D_stacked_cylinders_curved.nekg"),
        output=abs_path("ADR_3D_stacked_cylinders_curved.fld"),
        checkpoints=[abs_path("ADR_3D_stacked_cylinders_curved_0.chk")],
        filters=[],
        boundary_conditions=[],
    )

@NekUpload.testutils.dataset
@NekUpload.testutils.geometry_composite_info(
    geom_info=NekTestGeometryComposite(
        composite_id_to_definition_map= {1:"F[9-12,43-44,17,45,71,130-132,160-163,41]",6:" R[2,10-11,36,48]"},
        dim=3,
        description="Composite IDs are not sequential 1,6 rathre than 1,2"
    )
)
def advection_3D_prism():
    return NekTestDataset(
        solver_type=SolverType.ADR_SOLVER,
        session=abs_path("advection_3D_prism.xml"),
        geometry=abs_path("advection_3D_prism.nekg"),
        output=abs_path("advection_3D_prism.fld"),
        checkpoints=[],
        filters=[],
        boundary_conditions=[]
    )

@NekUpload.testutils.dataset
@NekUpload.testutils.geometry_composite_info(
    geom_info=NekTestGeometryComposite(
        composite_id_to_definition_map={0:"H[0-7]",1:"F[0,11,20,29]",2:"F[1,6,21,25]",
            3:"F[22,26,30,33]",4:"F[13,17,31,34]",5:"F[4,9,14,18]",6:"F[10,19,28,35]"
        },
        dim=3
    )
)
def Helmholtz3D_Cube_BndFile_Lagrange():
    return NekTestDataset(
        solver_type=SolverType.ADR_SOLVER,
        session=abs_path("Helmholtz3D_Cube_BndFile_Lagrange.xml"),
        geometry=abs_path("Helmholtz3D_Cube_BndFile_Lagrange.nekg"),
        output=abs_path("Helmholtz3D_Cube_BndFile_Lagrange.fld"),
        checkpoints=[],
        filters=[],
        boundary_conditions=[abs_path("Helmholtz3D_Cube_BndFile_Lagrange.bc")]
    )

if __name__=="__main__":
    from NekUpload.testutils import GEOMETRY_COMPOSITE_DATASETS
    print(GEOMETRY_COMPOSITE_DATASETS)
    print(GEOMETRY_COMPOSITE_DATASETS[1][1].boundary_composite_id_to_definition_map)