from pathlib import Path
from NekUpload.testutils import NekTestDataset,NekTestGeometryComposite
import NekUpload.testutils
from NekUpload.NekData.data_type import SolverType

def abs_path(file: str) -> str:
    absolute_path = Path(__file__).resolve().parent / file
    return str(absolute_path)

@NekUpload.testutils.dataset
@NekUpload.testutils.geometry_composite_info(
    geom_info=NekTestGeometryComposite(
        composite_id_to_definition_map={1:"E[0,6,45,47]",2:"E[12,23,20,25]",
        3:"E[4,9,16,15]",4:"E[26,28,37,39]",5:"Q[0-7]",6:"T[8-23]"
        },
        dim=2,
        description="Change ordering of some composite element definitions"
    )
)
def ChanFlow_3DH1D_pRef():
    return NekTestDataset(
        solver_type=SolverType.INCOMPRESSIBLE_NAVIER_STOKES_SOLVER,
        session=abs_path("ChanFlow_3DH1D_pRef.xml"),
        geometry=abs_path("ChanFlow_3DH1D_pRef.nekg"),
        output=abs_path("ChanFlow_3DH1D_pRef.fld"),
        checkpoints=[abs_path("ChanFlow_3DH1D_pRef_0.chk"),abs_path("ChanFlow_3DH1D_pRef_1.chk")],
        filters=[],
        boundary_conditions=[]
    )

@NekUpload.testutils.dataset
@NekUpload.testutils.geometry_composite_info(
    geom_info=NekTestGeometryComposite(
        composite_id_to_definition_map={0:"Q[0-3]",2:"E[2,7]",3:"E[4,9]",1:"E[0-1,10-11]",},
        dim=2,
        description="Change ordering of some composite element definitions"
    )
)
def ChanFlow_m8():
    return NekTestDataset(
        solver_type=SolverType.INCOMPRESSIBLE_NAVIER_STOKES_SOLVER,
        session=abs_path("ChanFlow_m8.xml"),
        geometry=abs_path("ChanFlow_m8.nekg"),
        output=abs_path("ChanFlow_m8.fld"),
        checkpoints=[abs_path("ChanFlow_m8_0.chk"),abs_path("ChanFlow_m8_1.chk")],
        filters=[abs_path("ChanFlow_m8_max.fld")],
        boundary_conditions=[]
    )

@NekUpload.testutils.dataset
@NekUpload.testutils.geometry_composite_info(
    geom_info=NekTestGeometryComposite(
        composite_id_to_definition_map={0:"E[0-8,142-152,307-312,397-407,608-616,1023,1022]",
            1:" E[1834,1837,1840,1843] ",2:"E[1821,1824,1827,1830]",
            3:"E[1831,1846,1849,1852,1832]",4:"E[1819,1853,1856,1859,1844]",
            5:"Q[0-1330]",6:"T[1331-1436]"
        },
        dim=2,
        description="Some extra whitespace, but should still work"
    )
)
def PlungingAirfoil_3DH1D_with_NACA0012_Re400():
    return NekTestDataset(
        solver_type=SolverType.INCOMPRESSIBLE_NAVIER_STOKES_SOLVER,
        session=abs_path("PlungingAirfoil_3DH1D_with_NACA0012_Re400.xml"),
        geometry=abs_path("PlungingAirfoil_3DH1D_with_NACA0012_Re400.nekg"),
        output=abs_path("PlungingAirfoil_3DH1D_with_NACA0012_Re400.fld"),
        checkpoints=[abs_path("PlungingAirfoil_3DH1D_with_NACA0012_Re400_0.chk")],
        filters=[],
        boundary_conditions=[]
    )

@NekUpload.testutils.dataset
@NekUpload.testutils.geometry_composite_info(
    geom_info=NekTestGeometryComposite(
        composite_id_to_definition_map={0:" A[0-47]",1:" F[0,19,34,49,64,83,94,109] ",
            2:"F[1,4,10,13,65,68,74,78]",3:"F[69,72,75,81,95,97,102,105]",
            4:"F[50,52,57,61,110,112,115,117]",5:"F[20,23,27,29,53,56,58,63]",
            6:"F[14,30,45,60,77,91,106,118]"
        },
        dim=3,
        description="Some extra whitespace, but should still work"
    )
)
def Tet_channel_robin_m3():
    return NekTestDataset(
        solver_type=SolverType.INCOMPRESSIBLE_NAVIER_STOKES_SOLVER,
        session=abs_path("Tet_channel_robin_m3.xml"),
        geometry=abs_path("Tet_channel_robin_m3.nekg"),
        output=abs_path("Tet_channel_robin_m3.fld"),
        checkpoints=[abs_path("Tet_channel_robin_m3_0.chk")],
        filters=[],
        boundary_conditions=[]
    )

@NekUpload.testutils.dataset
@NekUpload.testutils.geometry_composite_info(
    geom_info=NekTestGeometryComposite(
        composite_id_to_definition_map={0:"Q[0-3]",1:"E[0,7]",2:"E[6,3]",3:"E[8,10]",4: "E[11,5]"},
        dim=2,
        description="Uses new integration time scheme method"
    )
)
def KovaFlow_VCSWeakPressure_TaylorHood():
    return NekTestDataset(
        solver_type=SolverType.INCOMPRESSIBLE_NAVIER_STOKES_SOLVER,
        session=abs_path("KovaFlow_VCSWeakPressure_TaylorHood.xml"),
        geometry=abs_path("KovaFlow_VCSWeakPressure_TaylorHood.nekg"),
        output=abs_path("KovaFlow_VCSWeakPressure_TaylorHood.fld"),
        functions=[abs_path("KovaFlow_VCSWeakPressure_TaylorHood.rst")] 
    )

@NekUpload.testutils.dataset
@NekUpload.testutils.geometry_composite_info(
    geom_info=NekTestGeometryComposite(
        composite_id_to_definition_map={0:"Q[0-3]",1:"E[0-1,10-11]",2:"E[2,7]",3:"E[4,9]",4: "E[5]"},
        dim=2,
        description="Uses new integration time scheme method"
    )
)
def ChanFlow2D_bcsfromfiles():
    return NekTestDataset(
        solver_type=SolverType.INCOMPRESSIBLE_NAVIER_STOKES_SOLVER,
        session=abs_path("ChanFlow2D_bcsfromfiles.xml"),
        geometry=abs_path("ChanFlow2D_bcsfromfiles.nekg"),
        output=abs_path("ChanFlow2D_bcsfromfiles.fld"),
        checkpoints=[abs_path("ChanFlow2D_bcsfromfiles_0.chk")],
        boundary_conditions=[abs_path("ChanFlow2D_bcsfromfiles_u1_0.bc"),
                            abs_path("ChanFlow2D_bcsfromfiles_u3_0.bc"),
                            abs_path("ChanFlow2D_bcsfromfiles_v3_0.bc")],
        functions=[abs_path("ChanFlow2D_bcsfromfiles_uforce.fld")]
    )