import pytest
from NekUpload.metadata import *

###############################################################################
# Test entire workflow involved with Metadata
###############################################################################


@pytest.mark.integration
def test_metadata_instantiation():
    title = "Armistice Day"
    publication_date = "1918-11-11"
    org1_name = "Axis Alliance"
    org2_name = "Entente Alliance"
    org1 = InvenioOrgInfo(org1_name)
    org2 = InvenioOrgInfo(org2_name)
    version = "v1.0.0"
    description = "Peace in our time"
    publisher = "N/A"
    author_list: list[InvenioOrgInfo] = [org1,org2]

    metadata = InvenioMetadata(title,publication_date,author_list,ResourceType.DATASET)
    metadata.add_version(version)
    metadata.add_description(description)
    metadata.add_publisher(publisher)

    metadata_payload = metadata.get_metadata_payload()
    
    assert metadata_payload.get("title",None) == title, f"Expected {title}, but found {metadata_payload.get('title',None)}"
    assert metadata_payload.get("publication_date",None) == publication_date, f"Expected {publication_date}, but found {metadata_payload.get('publication_date',None)}"
    assert metadata_payload.get("version",None) == version, f"Expected {version}, but found {metadata_payload.get('version',None)}"
    assert metadata_payload.get("publisher",None) == publisher, f"Expected {publisher}, but found {metadata_payload.get('publisher',None)}"
    assert metadata_payload.get("description",None) == description, f"Expected {description}, but found {metadata_payload.get('description',None)}"
    
    #check creator via InvenioUserInfo.get_info()
    org1_expected_info = org1.get_info()
    org2_expected_info = org2.get_info()
    expected_author_list_info = [{"person_or_org": org1_expected_info},{"person_or_org": org2_expected_info}]

    assert metadata_payload.get("creators",None) == expected_author_list_info, f"Expected {expected_author_list_info}, but found {metadata_payload.get('creators',None)}"