import pytest
import requests

from NekUpload.utils import gitlab_api as GitlabAPI
import random
import string

###############################################################################
# Too lazy to mock
# So integration test with NekUpload repository and Nektar repository
# Test mostly with NekUpload rather than Nektar
###############################################################################

def is_repository_available(url:str) -> bool:
    try:
        response = requests.get(url,timeout=5)
        return response.status_code == 200
    except requests.RequestException:
        return False

GITLAB_NEKTAR_INFO_HOST = "https://gitlab.nektar.info"
IS_GITLAB_NEKTAR_INFO_AVAILABLE = is_repository_available(GITLAB_NEKTAR_INFO_HOST)
NEKUPLOAD_PROJECT_ID = "396"
NEKTAR_PROJECT_ID = "2"

@pytest.mark.integration
@pytest.mark.internet
@pytest.mark.parametrize(
    "hostname, project_id, gitsha",
    [
        pytest.param(
            GITLAB_NEKTAR_INFO_HOST,
            NEKUPLOAD_PROJECT_ID,
            "0ee9c83e5cf0b907574cf0165977335fcf77a1ac",
            marks=pytest.mark.skipif(
                not IS_GITLAB_NEKTAR_INFO_AVAILABLE,
                reason=f"{GITLAB_NEKTAR_INFO_HOST} not available.",
            ),
        ),
        pytest.param(
            GITLAB_NEKTAR_INFO_HOST,
            NEKUPLOAD_PROJECT_ID,
            "3c97d6d", #test shortedned gitsha
            marks=pytest.mark.skipif(
                not IS_GITLAB_NEKTAR_INFO_AVAILABLE,
                reason=f"{GITLAB_NEKTAR_INFO_HOST} not available.",
            ),
        ),
        pytest.param(
            GITLAB_NEKTAR_INFO_HOST,
            NEKTAR_PROJECT_ID,
            "ebf2aec4f840729ffb2845ead6d462be6f6f341a",
            marks=pytest.mark.skipif(
                not IS_GITLAB_NEKTAR_INFO_AVAILABLE,
                reason=f"{GITLAB_NEKTAR_INFO_HOST} not available.",
            ),
        ),
    ],
)
def test_get_valid_commit(hostname:str,project_id:str,gitsha:str):
    response:requests.Response = GitlabAPI.get_single_commit(hostname,project_id,gitsha)

    COMMIT_IS_IN_REPO_RESPONSE = 200
    assert response.status_code == COMMIT_IS_IN_REPO_RESPONSE

@pytest.mark.integration
@pytest.mark.internet
@pytest.mark.parametrize(
    "hostname, project_id",
    [
        pytest.param(
            GITLAB_NEKTAR_INFO_HOST,
            NEKUPLOAD_PROJECT_ID,
            marks=pytest.mark.skipif(
                not IS_GITLAB_NEKTAR_INFO_AVAILABLE,
                reason=f"{GITLAB_NEKTAR_INFO_HOST} not available.",
            ),
        ),
        pytest.param(
            GITLAB_NEKTAR_INFO_HOST,
            NEKTAR_PROJECT_ID,
            marks=pytest.mark.skipif(
                not IS_GITLAB_NEKTAR_INFO_AVAILABLE,
                reason=f"{GITLAB_NEKTAR_INFO_HOST} not available.",
            ),
        ),
    ],
)
def test_get_invalid_commit(hostname:str,project_id:str):
    #first get all valid commits in repository
    response = GitlabAPI.get_list_of_commits(hostname,project_id)
    data = response.json()
    valid_commits = set([commit["id"] for commit in data])

    def generate_random_hex_hash():
        return ''.join(random.choices(string.hexdigits.lower(), k=40))

    gitsha: str = generate_random_hex_hash()
    while gitsha in valid_commits:
        gitsha = generate_random_hex_hash()

    with pytest.raises(GitlabAPI.MissingGitlabCommit):
        GitlabAPI.get_single_commit(hostname,project_id,gitsha)